package com.jasamedika.medifirst2000.service;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.jasamedika.medifirst2000.entities.PapMata;
import com.jasamedika.medifirst2000.vo.PapMataVO;

/**
 * PAPMata Service
 * 
 * @author Askur
 */
public interface PapMataService<T> extends BaseVoService<PapMata, PapMataVO, String> {
	Map<String,Object> addMata(PapMataVO vo);
	Map<String,Object> updateMata(PapMataVO vo);
	Map<String,Object> findByNoCmAndTglInput(String noCm,Date tglInput);
	List<Map<String,Object>> findByTglInput(String startDate,String endDate);
	List<Map<String,Object>> findListData(String jenis);
	List<Map<String,Object>> findStreak();
	List<Map<String,Object>> findKategoriMata();
	Map<String,Object> findRegistrasiPelayananPasienByNoCmAndTglRegistrasi(String noCm, short kdProfile,Date tglRegistrasi);
	Map<String,Object> findPapMataByNoRec(String noRec);
	public Map<String,Object> findPasienByNoCm(String noCm);
}
